from flask import current_app, request, jsonify, make_response
from thermodynamics.mix.equilibrium import solve_eos, bubble_temperature, bubble_pressure, dew_temperature, dew_pressure
from thermodynamics.mix.binaryDiagrams import pressure_composition, temperature_composition
from ..helpers.substanceHelpers import getSubstanceData
from numpy import round

def solve():
    req = request.get_json()
    temperature, pressure, tc, pc, acentric, liq_compositions, vap_compositions, kij, heat_capacity_constants, method, alfa_function, substances = get_input_data(req)
    phase = req['phase']
    result = solve_eos(
        temperature,
        pressure,
        tc,
        pc,
        acentric,
        liq_compositions,
        vap_compositions,
        kij,
        method = method,
        alfa_function = alfa_function,
        properties = True,
        heat_capacities = heat_capacity_constants
        )

    if (phase == 'liq'):
        response = {
            "phase":"Liquid",
            "fugacity":list(round(result['liq_fugacity'], 4)),
            "enthalpy":list(round(result['enthalpy_liq'], 2)),
            "entropy":list(round(result['entropy_liq'], 2)),
            "z":float(round(result['z_liq'], 4)),
            "compositions": list(liq_compositions)
        }
    else:
        response = {
            "phase":"Vapor",
            "fugacity":list(round(result['vap_fugacity'], 4)),
            "enthalpy":list(round(result['enthalpy_vap'], 2)),
            "entropy":list(round(result['entropy_vap'], 2)),
            "z":float(round(result['z_vap'], 4)),
            "compositions": list(vap_compositions)
        }

    response["temperature"] = float(round(temperature, 4))
    response["pressure"] = float(round(pressure, 4))
    response["substances"] = substances

    res = make_response(jsonify(response), 200)

    return res

def get_input_data(req):
    temperature = float(req['temperature']) if 'temperature' in req else None
    pressure = float(req['pressure']) if 'pressure' in req else None
    method = req['eos']
    alfa_function = req['alfa']
    compositions = [float(req['first_composition']) if 'first_composition' in req else None, float(req['second_composition']) if 'second_composition' in req else None]
    substance_1, tc_1, pc_1, acentric_1, heat_capacity_constants_1 = getSubstanceData(req['first_substance'])
    substance_2, tc_2, pc_2, acentric_2, heat_capacity_constants_2 = getSubstanceData(req['second_substance'])

    if (req['k12']):
        k12 = float(req['k12'])
        k21 = float(req['k21'])
        kij = [[k12, k21], [k21, k12]]
    else:
        kij = [[0,0], [0,0]]

    return (temperature, pressure, [tc_1,tc_2], [pc_1,pc_2], [acentric_1, acentric_2], compositions, compositions, kij, [heat_capacity_constants_1, heat_capacity_constants_2], method, alfa_function, [substance_1.name, substance_2.name])

def solve_bubble_temperature():
    req = request.get_json()
    temperature, pressure, tc, pc, acentric, liq_compositions, vap_compositions, kij, heat_capacity_constants, method, alfa_function, substances = get_input_data(req)
    phase = req['phase']
    new_t, p, liq_compositions, new_vap_compositions = bubble_temperature(temperature, pressure, tc, pc, acentric, liq_compositions, vap_compositions, kij, method=method, alfa_function=alfa_function)

    result = solve_eos(
        new_t,
        pressure,
        tc,
        pc,
        acentric,
        liq_compositions,
        new_vap_compositions,
        kij,
        method = method,
        alfa_function = alfa_function,
        properties = True,
        heat_capacities = heat_capacity_constants
        )

    if (temperature == new_t):
        response = {"message":"Set a new temperature approximation"}
    else :
        response = {
            "liquid" : {
                "phase":"Liquid",
                "fugacity":list(round(result['liq_fugacity'], 4)),
                "enthalpy":list(round(result['enthalpy_liq'], 2)),
                "entropy":list(round(result['entropy_liq'], 2)),
                "z":float(round(result['z_liq'], 4)),
                "compositions": list(round(liq_compositions, 4)),
                "temperature" : float(round(new_t, 4)),
                "pressure":float(round(p, 4)),
                "substances":substances
            },
            "vapor" : {
                "phase":"Vapor",
                "fugacity":list(round(result['vap_fugacity'], 4)),
                "enthalpy":list(round(result['enthalpy_vap'], 2)),
                "entropy":list(round(result['entropy_vap'], 2)),
                "z":float(round(result['z_vap'], 4)),
                "compositions": list(round(new_vap_compositions, 4)),
                "temperature" : float(round(new_t, 4)),
                "pressure":float(round(p, 4)),
                "substances":substances
            }
        }

    res = make_response(jsonify(response), 200)

    return res

def solve_bubble_pressure():
    req = request.get_json()
    temperature, pressure, tc, pc, acentric, liq_compositions, vap_compositions, kij, heat_capacity_constants, method, alfa_function, substances = get_input_data(req)
    phase = req['phase']
    t,new_p,liq_compositions,new_vap_compositions = bubble_pressure(temperature, pressure, tc, pc, acentric, liq_compositions, vap_compositions, kij, method=method, alfa_function=alfa_function)
    
    result = solve_eos(
        t,
        new_p,
        tc,
        pc,
        acentric,
        liq_compositions,
        new_vap_compositions,
        kij,
        method = method,
        alfa_function = alfa_function,
        properties = True,
        heat_capacities = heat_capacity_constants
        )

    if (pressure == new_p):
        response = {"message":"Set a new pressure approximation"}
    else :
        response = {
            "liquid" : {
                "phase":"Liquid",
                "fugacity":list(round(result['liq_fugacity'], 4)),
                "enthalpy":list(round(result['enthalpy_liq'], 2)),
                "entropy":list(round(result['entropy_liq'], 2)),
                "z":float(round(result['z_liq'], 4)),
                "compositions": list(round(liq_compositions, 4)),
                "temperature" : float(round(t, 4)),
                "pressure":float(round(new_p, 4)),
                "substances":substances
            },
            "vapor" : {
                "phase":"Vapor",
                "fugacity":list(round(result['vap_fugacity'], 4)),
                "enthalpy":list(round(result['enthalpy_vap'], 2)),
                "entropy":list(round(result['entropy_vap'], 2)),
                "z":float(round(result['z_vap'], 4)),
                "compositions": list(round(new_vap_compositions, 4)),
                "temperature" : float(round(t, 4)),
                "pressure":float(round(new_p, 4)),
                "substances":substances
            }
        }

    res = make_response(jsonify(response), 200)

    return res

def solve_dew_temperature():
    req = request.get_json()
    temperature, pressure, tc, pc, acentric, liq_compositions, vap_compositions, kij, heat_capacity_constants, method, alfa_function, substances = get_input_data(req)
    phase = req['phase']

    new_t,p,new_liq_compositions,vap_compositions = dew_temperature(temperature, pressure, tc, pc, acentric, liq_compositions, vap_compositions, kij, method=method, alfa_function=alfa_function)

    result = solve_eos(
        new_t,
        p,
        tc,
        pc,
        acentric,
        new_liq_compositions,
        vap_compositions,
        kij,
        method = method,
        alfa_function = alfa_function,
        properties = True,
        heat_capacities = heat_capacity_constants
        )

    if (temperature == new_t):
        response = {"message":"Set a new temperature approximation"}
    else :
        response = {
            "liquid" : {
                "phase":"Liquid",
                "fugacity":list(round(result['liq_fugacity'], 4)),
                "enthalpy":list(round(result['enthalpy_liq'], 2)),
                "entropy":list(round(result['entropy_liq'], 2)),
                "z":float(round(result['z_liq'], 4)),
                "compositions": list(round(new_liq_compositions, 4)),
                "temperature" : float(round(new_t, 4)),
                "pressure":float(round(p, 4)),
                "substances":substances
            },
            "vapor" : {
                "phase":"Vapor",
                "fugacity":list(round(result['vap_fugacity'], 4)),
                "enthalpy":list(round(result['enthalpy_vap'], 2)),
                "entropy":list(round(result['entropy_vap'], 2)),
                "z":float(round(result['z_vap'], 4)),
                "compositions": list(round(vap_compositions, 4)),
                "temperature" : float(round(new_t, 4)),
                "pressure":float(round(p, 4)),
                "substances":substances
            }
        }

    res = make_response(jsonify(response), 200)

    return res

def solve_dew_pressure():
    req = request.get_json()
    temperature, pressure, tc, pc, acentric, liq_compositions, vap_compositions, kij, heat_capacity_constants, method, alfa_function, substances = get_input_data(req)
    phase = req['phase']

    t,new_p,new_liq_compositions,vap_compositions = dew_pressure(temperature, pressure, tc, pc, acentric, liq_compositions, vap_compositions, kij, method=method, alfa_function=alfa_function)

    result = solve_eos(
        t,
        new_p,
        tc,
        pc,
        acentric,
        new_liq_compositions,
        vap_compositions,
        kij,
        method = method,
        alfa_function = alfa_function,
        properties = True,
        heat_capacities = heat_capacity_constants
        )

    if (pressure == new_p):
        response = {"message":"Set a new pressure approximation"}
    else :
        response = {
            "liquid" : {
                "phase":"Liquid",
                "fugacity":list(round(result['liq_fugacity'], 4)),
                "enthalpy":list(round(result['enthalpy_liq'], 2)),
                "entropy":list(round(result['entropy_liq'], 2)),
                "z":float(round(result['z_liq'], 4)),
                "compositions": list(round(new_liq_compositions, 4)),
                "temperature" : float(round(t, 4)),
                "pressure":float(round(new_p, 4)),
                "substances":substances
            },
            "vapor" : {
                "phase":"Vapor",
                "fugacity":list(round(result['vap_fugacity'], 4)),
                "enthalpy":list(round(result['enthalpy_vap'], 2)),
                "entropy":list(round(result['entropy_vap'], 2)),
                "z":float(round(result['z_vap'], 4)),
                "compositions": list(round(vap_compositions, 4)),
                "temperature" : float(round(t, 4)),
                "pressure":float(round(new_p, 4)),
                "substances":substances
            }
        }

    res = make_response(jsonify(response), 200)

    return res

def pressureCompositionDiagram():
    req = request.get_json()
    temperature, pressure, tc, pc, acentric, liq_compositions, vap_compositions, kij, heat_capacity_constants, method, alfa_function, substances = get_input_data(req)

    bubble_line, dew_line = pressure_composition(temperature, tc, pc, acentric, kij, method=method, alfa_function=alfa_function, atomo4=True)

    response = {
        "bubble_line" : bubble_line,
        "dew_line" : dew_line
    }

    res = make_response(jsonify(response), 200)
    return res

def temperatureCompositionDiagram():
    req = request.get_json()
    temperature, pressure, tc, pc, acentric, liq_compositions, vap_compositions, kij, heat_capacity_constants, method, alfa_function, substances = get_input_data(req)

    bubble_line, dew_line = temperature_composition(pressure, tc, pc, acentric, kij, method=method, alfa_function=alfa_function, atomo4=True)

    response = {
        "bubble_line" : bubble_line,
        "dew_line" : dew_line
    }

    res = make_response(jsonify(response), 200)
    return res
