'use strict';

import { API_URL } from "../../constants";
import Plotly from 'plotly.js-basic-dist-min';

class PressureTemperatureDiagram {
    constructor() {
        this.extra_substance = 2;
    }

    addSubstance() {
        const substances_div = document.getElementById('substances_element');
        const substances_dropdown = document.getElementById('substances_dropdown');
        const molar_compositions_div = document.getElementById('molar_compositions_element');
        const new_dropdown = substances_dropdown.cloneNode(true);

        this.extra_substance += 1;

        // Substance selection
        new_dropdown.id = null;
        new_dropdown.classList.remove('d-none');
        new_dropdown.firstElementChild.name = 'substances[]';
        new_dropdown.firstElementChild.classList.add('new-select2');
        substances_div.appendChild(new_dropdown);
        $('.new-select2').select2({
            theme:"bootstrap4"
        });

        // Molar composition
        const div = document.createElement('div');
        const input = document.createElement('input');
        const del_button = document.createElement('button');
        const trash_icon = document.createElement('i');

        div.classList.add('d-flex', 'justify-content-between');
        input.classList.add('form-control', 'mb-2');
        input.setAttribute("type", "number");
        input.setAttribute("min","0");
        input.setAttribute("max", "1");
        input.setAttribute("name", "molar_compositions[]");
        input.setAttribute("placeholder", "Molar composition");
        input.style.width = "90%";
        del_button.id = 'extra_substance_button_' + this.extra_substance;
        del_button.classList.add('btn', 'btn-danger', 'btn-circle', 'btn-sm');
        trash_icon.classList.add('fas', 'fa-trash');
        
        // Append to div
        del_button.appendChild(trash_icon);
        div.appendChild(input);
        div.appendChild(del_button);

        // Delete listener
        del_button.addEventListener('click', (event) => {
            event.preventDefault();
            div.remove();
            new_dropdown.remove();
            this.extra_substance -= 1;
        });

        molar_compositions_div.appendChild(div);
    }

    addBinaryInteraction() {
        const add_btn = document.getElementById('add_kij');
        const del_btn = document.getElementById('del_kij');
        const div = document.getElementById('binary_interaction_input');

        add_btn.classList.add('d-none');
        del_btn.classList.remove('d-none');

        this.createKijTable(div);
    }

    deleteBinaryInteraction() {
        const add_btn = document.getElementById('add_kij');
        const del_btn = document.getElementById('del_kij');
        const kij_table = document.getElementById('kij_table');

        add_btn.classList.remove('d-none');
        del_btn.classList.add('d-none');
        kij_table.remove();
    }

    createKijTable(div) {
        const table = document.createElement('table');
        const table_body = document.createElement('tbody');
        const header_row = document.createElement('tr');
        table.id = 'kij_table';

        let th;
        for (let i = 0; i <= this.extra_substance; i++) {
            th = document.createElement('th');
            th.innerText = i;
            header_row.appendChild(th);
        }

        table_body.appendChild(header_row);

        for (let i = 1; i <= this.extra_substance; i++) {
            let tr = document.createElement('tr');
            let td = document.createElement('td');
            td.innerText = i;
            tr.appendChild(td);
            for (let j = 1; j <= this.extra_substance; j++) {
                let td_input = document.createElement('td');
                let input = document.createElement('input');
                input.classList.add('form-control');
                input.setAttribute("type", "number");
                input.setAttribute("name", "kij[]");
                input.value = '0.0';

                if (i==j) {
                    input.disabled = true;
                }

                td_input.appendChild(input);
                tr.appendChild(td_input);
            }
            table_body.appendChild(tr);
        }

        table.appendChild(table_body);
        div.appendChild(table);
    }

    async configureButtons() {
        const liq_phase = document.getElementById('liq_phase').checked;
        const vap_phase = document.getElementById('vap_phase').checked;
        const draw_btn = document.getElementById('draw');

        if (liq_phase || vap_phase) {
            draw_btn.disabled = false;
        }
    }

    getInputData() {
        const eos = document.getElementById('eos').value;
        const alfa = document.getElementById('alfa').value;
        const liq_phase = document.getElementById('liq_phase').checked;
        const vap_phase = document.getElementById('vap_phase').checked;
        const substances_form = document.getElementsByName('substances[]');
        const compositions_form = document.getElementsByName('molar_compositions[]');
        const kij_form = document.getElementsByName('kij[]');
        
        let phase;
        if (liq_phase) {
            phase = 'liq';
        } else if (vap_phase) {
            phase = 'vap';
        }

        let substances = [];
        let compositions = [];
        let kij = [];

        for(let element of substances_form.values()) { 
            substances.push(element.value);
        }

        for(let element of compositions_form.values()) {
            compositions.push(element.value);
        }

        for(let element of kij_form.values()) {
            kij.push(element.value);
        }

        return {eos, alfa, phase, substances, compositions, kij};
    }

    async render() {
        const url = API_URL + '/mixture/pressure-temperature';
        const data = this.getInputData();
        const response = await fetch(url, {
            method: "POST",
            body: JSON.stringify(data),
            mode: 'cors',
            headers: new Headers({
                "Content-Type": "application/json"
            }),
            cache: 'default',
        });

        const body = await response.json();
        const solution_ele = document.getElementById('solution');
        const card_title = document.getElementById('solution_title');
        card_title.innerText = 'Pressure temperature diagram';

        const bubble_line = {
            x: body.bubble_line[0],
            y: body.bubble_line[1],
            mode: 'markers',
            type: 'scatter',
            name: 'Bubble line'
        };

        const dew_line = {
            x: body.dew_line[0],
            y: body.dew_line[1],
            mode: 'markers',
            type: 'scatter',
            name: 'Dew line'
        }

        const points = [bubble_line, dew_line];
        const layout = {
            title: 'Pressure temperature diagram',
            xaxis: {
                title: {
                    text: 'Temperature [K]',
                }
            },
            yaxis: {
                title: {
                    text: 'Pressure [bar]',
                }
            }
        }

        const config = { responsive: true }

        Plotly.newPlot('pt_diagram', points, layout, config);
        solution_ele.classList.remove('d-none');

    }

    async download_pt_csv() {
        const url = API_URL + '/mixture/pressure-temperature';
        const data = this.getInputData();
        const response = await fetch(url, {
            method: "POST",
            body: JSON.stringify(data),
            mode: 'cors',
            headers: new Headers({
                "Content-Type": "application/json"
            }),
            cache: 'default',
        });

        const body = await response.json();
        let csvContent = "data:text/csv;charset=utf-8,bubble_temperature,bubble_pressure,dew_temperature,dew_pressure\r\n";

        for (let i = 0; i < body.bubble_line[0].length; i++) {
            const bubble_temperature = body.bubble_line[0][i];
            const bubble_pressure = body.bubble_line[1][i];
            const dew_temperature = body.dew_line[0][i];
            const dew_pressure = body.dew_line[1][i];

            csvContent += await bubble_temperature + ',' + bubble_pressure + ',' + dew_temperature + ',' + dew_pressure + "\r\n";
            
        }

        const encodedUri = encodeURI(csvContent);
        const link = document.createElement("a");
        link.setAttribute("href", encodedUri);
        link.setAttribute("download", 'Pressure Temperature diagram.csv');
        document.body.appendChild(link); // Required for FF

        link.click();
    }

    configureAjaxCalls() {
        document.getElementById('add_substance').addEventListener('click', this.addSubstance.bind(this));
        document.getElementById('add_kij').addEventListener('click', this.addBinaryInteraction.bind(this));
        document.getElementById('del_kij').addEventListener('click', this.deleteBinaryInteraction.bind(this));
        document.getElementById('liq_phase').addEventListener('change', this.configureButtons.bind(this));
        document.getElementById('vap_phase').addEventListener('change', this.configureButtons.bind(this));
        document.getElementById('draw').addEventListener('click', this.render.bind(this));
        document.getElementById('download_pt_csv').addEventListener('click', this.download_pt_csv.bind(this));
    }
}

export const pressure_temperature_diagram = new PressureTemperatureDiagram();
